/*  Lindley Systems Form Filler Program

*/
/* FATAL Error Numbers: See FATALERR.DOC */

#define MAIN_PROG
#include "globals.h"
#include <signal.h>
#include <stdlib.h>
#include <process.h>
#include <time.h>
#include <ctype.h>

extern int rows;

void center (int line, char *text);
void fkeydefn (int line, int col, int fkey, char *text);
void give_error(char *text);
void display_status();
void put_box_char (int x, int y, char ch);
void draw_a_box(int y1, int x1, int y2, int x2, int dflag);

int save_nor;

#ifdef DEMO
static char disabled[]="Feature Disabled in Demo Version";
#endif

void center (int line, char *text)
{
   char *p, buf[80];
   int i;

	p = text;
	while (isspace(*p)) p++;
	strcpy(buf,p);
	i = strlen(buf);
	p = buf + i;
	while (i-- > 0 && isspace(*--p)) *p = '\0';

	setcur(line,40-(strlen(buf)/2));
	wstr(buf,1);
	wstr("\r\n",1);
}

void fkeydefn (int line, int col, int fkey, char *text)
{  char buf[10];
	setcur(line,col);
	if (fkey <= 9)
	  wchar(' ');
	sprintf(buf,"\1F%d\2 ",fkey);
	wstr(buf,1);
	wstr(text,1);
}

void give_error(char *text)
{  static char _stars[]=" * * * ";

	stat_line = stat_msg;
	strcpy(stat_line,_stars);
	strcat(stat_line,text);
	strcat(stat_line,_stars);
}

void display_status()
{
	setcur(win_lr + 3, 0);
	wchar('\2');			/* Normal video */
	wstr(stat_line,1);
	clear_eol();
}

/*
	Draws a box in the current block.
	dflag tells us whether to draw a single (1) or double (2) box,
		or is 0 for erasing.
*/

void put_box_char(int x,int y,char ch)
{	setcur(y,x);
	wchar(ch);
}

void draw_a_box(int y1, int x1, int y2, int x2, int dflag)
{
    int temp, edge;
    char aline[82];
    static char corners [4] [6] = 
			      { {218, 192, 191, 217, 196, 179}, /* single */
				{201, 200, 187, 188, 205, 186}}; /* double */

	dflag--;	/* Array is 0-based */
	if ((y1 >= 0) && (y2 >= 0)) /* both line #'s defined? */
	  {	edge = 0;	/* assume not box, hence no edge */
		if (x1 > x2) { temp=x1; x1=x2; x2=temp; }
		if (y1 > y2) { temp=y1; y1=y2; y2=temp; }
		if ((y1 != y2) && (x1 != x2))	/* full-fledged Box! */
		    {	put_box_char(x1,y1,corners[dflag][0]);
			put_box_char(x1,y2,corners[dflag][1]);
			put_box_char(x2,y1,corners[dflag][2]);
			put_box_char(x2,y2,corners[dflag][3]);
			++edge;			/* lines drawn INSIDE box */
		    }
		if (x1 != x2)
		    {	temp = x2-x1+1-edge*2;
			memset(aline,corners[dflag][4],temp);
			aline[temp]='\0';
			setcur(y1,x1+edge);
			wstr(aline);
			setcur(y2,x1+edge);
			wstr(aline);
		    }
		if (y1 != y2)
		  for ( temp=y1+edge; temp<=y2-edge; ++temp)
		    {	put_box_char(x1,temp,corners[dflag][5]);
			put_box_char(x2,temp,corners[dflag][5]);
		    }
	  }
}

/* window_size sets the form-filling window to the normal region, plus
   the additional margins given (normally 0)
*/

void window_size(top_mgn,bottom_mgn)
{
    win_sr = top_mgn;
    win_lr = rows - BOT - bottom_mgn;
    win_nr = win_lr - win_sr + 1;
}

/* ---------------------------- main ---------------------------- */

main (argc,argv)
int argc;
char *argv[];

{
int c1, flag, i;
long tx;
    init_keys();
    give_error(no_form);
    memset(blanks,' ',80);
    memset(bgd,'_',30);
    bgd[30]=blanks[80]=*form_title = *form_name = '\0';
    strcpy(prn_name,Printer);

    /* guarantee that needed DOS services are available */
    if (signal(SIGINT, SIG_IGN) == (int(*)())-1)
    {   perror("Can't set SIGINT");
	exit(2);
    }
    /* flag == 0, show menu and get key */
    /* flag == 1, do not show menu, just get key */
    /* flag == 2, do not show menu, do not get key, use c1 */
    flag = (c1=initform(argc,argv)) ? 2: 0;
    if (!flag)
    {	cls();
	draw_a_box(4,20,14,60,2);
	center ( 5, program);
	center ( 6, version);
	center ( 7, copyright);
	center ( 8, "All Rights Reserved");

	put_box_char(20,9,199);
	put_box_char(60,9,182);
	draw_a_box(9,21,9,59,1);

	center (10, LS);
	center (11, "4257 Berwick Place");
	center (12, "Woodbridge, VA 22192");
	center (13, "(703) 590-8890");
	setcur(23,0);
	wstr(any_key,1);
	tx = clock() + 3*CLK_TCK;
	while (clock() < tx) if (kbhit()) {getch(); break;}
    }
/* Main program loop. */
    do
    {   if (!flag) 
	{   cls();
	    get_mode=1;

	    if (*form_title)
	      {	i = strlen(form_title)/2;
		draw_a_box(0,38-i,2,42+i,1);
		center(1,form_title);
	      }

	    draw_a_box(4,18,17,62,1);
	    setcur(5,20);
	    wstr(program,1);
	    wchar(' ');
	    wstr(version,1);
	    setcur(7,20);
	    wstr("Select function with indicated key:",1);
	    fkeydefn( 9,22, 1,"Help");
	    fkeydefn(10,22, 2,"Fill in Form");
	    fkeydefn(11,22, 3,"Select a Different Form");
	    fkeydefn(12,22, 4,"Select Form Print Destination");
	    fkeydefn(13,22, 5,"Save Current Form");
	    fkeydefn(14,22, 6,"Restore a Saved Form");
	    fkeydefn(15,22, 9,"Run DOS commands and return");
	    fkeydefn(16,22,10,"Terminate this program");

	    if (*form_name)
	      {	draw_a_box(18,24,21,56,2);
		setcur(19,26);
		wstr("\2Form file:  \1",1);
		wstr(form_name,1);
		setcur(20,26);
		wstr("\2Print to:   \1",1);
		wstr(prn_name,1);
	      }

	    display_status();

	}
	stat_line=empty;
	setcur(7,55);
	if (flag != 2) c1=gtkey();
	flag=0;
	switch(c1)
	{
/* F1 - Help */
	case F1:
	    help(helpname);
	    break;
/* F2 - Fill in form */
	case F2:
	    if (!form_file_open)
	    {   if (newform() != 0) break;
		cls();
		get_mode=1;
		setcur(0,0);
		wstr(form_title);
	    }
	    fillform();
	    break;
/* F3 - Select different form */
	case F3:
	    if (newform() != 0) break;
	    if (form_file_open) fillform();
	    break;
/* F4 - Change output file */
	case F4:
	    *temp2 = '\0'; /* Kill trash */
	    newout();
	    break;
/* F5 - Save current form */
	case F5:
	    if (form_file_open)
#ifdef DEMO
	      {	give_error(disabled);
		continue;
	      }
#else
		saveform();
#endif
	    else
	      {	give_error("No form to Save");
		continue;		/* Don't put up 'Press F3 or F6...' */
	      }
	    break;
/* F6 - Restore a saved form */
	case F6:
	    if (restform() != 0) break;
	    fillform();
	    break;
/* F9 - Execute DOS and return */
	case F9:
	    save_nor=nor_attr;
	    nor_attr=0x07;
	    wstr("\2 ",0);
	    cls();
	    printf("Use EXIT to return to %s.\n",program);
	    spawnlp(P_WAIT,Shell,Shell,NULL);
	    nor_attr=save_nor;
	    wstr("\2 ",0);
	    break;
/* F10 - Exit to DOS */
	case F10:
	    setcur(23,0);  /* Status line */
	    printf("Exit %s? (Y/N) ",program);
	    if ((c1 = gtkey() & 0x7F) == 'Y' || c1 == 'y') c1 = F10;
	    break;
/*  Anything else */
	default:
	    flag=1;
	}
	if (!form_file_open) give_error(no_form);
    }
    while (c1 != F10);
    exit_keys();
    cls();
    exit(0);
}
