#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "globals.h"

extern void update_value(struct field *afield);
extern int scrn_bot;	/* Screen line which is bottom of window */
extern int scrn_top;	/* Screen line which is top of window */
extern void give_error(char *text);
extern void display_status();
extern int keyword(char *keywords[], char **input);

static char *Str;
static int Error;


void recalc();
double parse (char *expression);
static double field_val (char *fieldname, int checking_val);
static double field_constant();
static double factor();
static double constant();
static double expr();
static double term();
static double atformula();

double parse (char *expression)
{   double rval;
	Error = 0;
	Str = expression;
	rval = expr();
	return (Error)?0:rval;
}

static double factor()
{   double rval=0, sign=1;

	if (*Str == '-')
	  { sign = -1; Str++; }
	if (*Str != '(')
	  rval = constant();
	else
	  { Str++;
	    rval = expr();
	    if (*Str == ')')
	      Str++;
	    else
	      { /* Missing close parenthesis */
	        Error++;
	      }
	  }
	return(rval * sign);
}

static double constant()
{   double rval=0, fr;
    int fraction=0;

	/* Handle @-Formulas */
	if (*Str == '@')
	  { Str++; return(atformula()); }

	if (!isdigit(*Str) && *Str != '.')
		return(field_constant());
	fraction = (*Str == '.');
	while (*Str && isdigit(*Str) && !fraction)
	  rval = (rval * 10) + (*Str++ - '0');
	if (*Str == '.')
	  { Str++;
	    fr = 0.1;
	    while (*Str && isdigit(*Str))
	      {	rval = rval + ((*Str++ - '0')*fr);
		fr = fr / 10.0;
	      }
	  }

	return(rval);
}

static double expr()
{
	char c;
	double left;

	left = term();
	while ( (c = *Str) == '+' || *Str == '-')
	  { Str++;
	    left = (c == '+')? left + term() : left - term();
	  }
	return(left);
}

static double term()
{	char c;
	double left, right;

	left = factor();
	while ( (c = *Str) == '*' || *Str == '/')
	  { Str++;
	    right= factor();
	    switch (c)
	      { case '*':
			left = left * right;
			break;
		case '/':
			if (right != 0)
			  left = left / right;
			else
			  Error++;
			break;
	      }
	  }
	return(left);
}

static double field_val(char *fieldname, int checking_value)
{   struct field *afield;
    char *Save;
    int ErrorCopy;

	afield = first_field -> next;
	while (afield != NULL)
	  { if (!strcmp(afield -> name,fieldname))
		if (checking_value)	/* Just want to know whether blank */
		  return(strcmp(afield->work,afield->mask)==0?0:1);
		else
	          switch (afield -> type)
		  { case 3:	/* Integer */
	            case 4:	/* Dollars */
	            case 5:	/* Real */
			if (afield -> formula != NULL && !afield -> ucaseonly)
			/* Formula, not yet recalc'd! */
			  { Save = Str; ErrorCopy=Error;
			    afield -> ucaseonly++;
			    afield -> value = parse(afield -> formula);
			    /* We may call ourselves recursively */
			    Str = Save; Error=ErrorCopy;
			  }
			return(afield -> value);
		  }
	    afield = afield -> next;
	  }
	return(0);
}

static double field_constant()
{
    int checking_value=0;
    char fname[20], *f;

	f = fname;
	if (*Str == '?')
	  { Str++; checking_value = 1; }

	while (isalnum(*Str))
		*f++ = *Str++;
	*f = '\0';
	return(field_val(fname,checking_value));
}

static double atformula()
{
    char formulatype[20], *f;
    int cond;
    double x, y, retval, maxval=9999999999;

	f = formulatype;
	while (isalnum(*Str))
		*f++ = *Str++;
	*f = '\0';
	if (*Str != '(' || !*++Str) return(0);
		/* Illegal syntax -- no open paren, or nothing after it */
	f = formulatype;
	switch (keyword(formula_keywords,&f))
	  { case 0:	/* Abs */
		x = expr();
		retval = (x<0)?-x:x;
		break;
	    case 1:	/* Avg */
		x = retval = 0;
		while (*Str && *Str != ')')
		  { while (*Str == ',') Str++;
		    if (*Str)
		      { retval += expr(); x++; }
		  }
		if (x)
		  retval /= x;
		else
		  retval = 0;
		break;
	    case 2:	/* Case */
		x = expr();
		cond=0;
		retval = 0;
		while (*Str && *Str != ')')
		  { while (*Str == ',')
		      { Str++; cond++; }	 /* One comma per argument */
		    y = expr();
		    if (cond == (int)x)
			retval = y;
		  }
		if (cond <= (int)x || (int)x <= 0)
			retval = y;	/* Last value is default */
		break;
	    case 3:	/* Count */
		x = 0;
		while (*Str && *Str != ')')
		  { while (*Str == ',') Str++;
		    if (*Str)
		      if (expr() != 0)
			x++;
		  }
		retval = x;
		break;
	    case 4:	/* If */
		x = expr();
		cond = *Str++ - 59;	/* 1 for <, 2 for =, 3 for > */
		if (*Str == '=')
		  { Str++; cond+=3; }	/* 4 for <=, 6 for >= */
		if (cond > 6 || cond < 0) cond=0;
		/* illegal condition? always return the FALSE value */
		y = expr();
		Str++;	/* Skip over comma */
		switch (cond)
		  { case 1:	/* < */
			cond = (x < y); break;
		    case 2:	/* = */
		    case 5:	/* ==, why not */
			cond = (x == y); break;
		    case 3:	/* > */
			cond = (x > y); break;
		    case 4:	/* <= */
			cond = (x <= y); break;
		    case 6:	/* >= */
			cond = (x >= y); break;
		  }
		x = expr();
		Str++;	/* Skip over comma */
		y = expr();
		retval = cond?x:y;	/* Take 1st expr if true, 2nd if false */
		break;
	    case 5:	/* Int */
		modf(expr(),&retval);
		break;
	    case 6:	/* Min */
		x = maxval;
		while (*Str && *Str != ')')
		  { while (*Str == ',') Str++;
		    if (*Str)
		      if ((y=expr()) < x)
			x = y;
		  }
		retval = x;
		break;
	    case 7:	/* Max */
		x = -maxval;
		while (*Str && *Str != ')')
		  { while (*Str == ',') Str++;
		    if (*Str)
		      if ((y=expr()) > x)
			x = y;
		  }
		retval = x;
		break;
	  }
	while (*Str && *Str++ != ')') ;
	return(retval);
}

void recalc()
{    struct field *afield;
     int scrn_line;

	/* We're going to use the Uppercase flag to determine which
	   fields we've already recalculated & which ones we haven't.
	   We know only the values of the non-formula fields.
	   ucaseonly is TRUE if value of field is known. */

	give_error("Recalculating");
	display_status();
	afield = first_field -> next;
	while (afield != NULL)
	  { if (afield -> formula != NULL)
		afield -> ucaseonly = 0;
	    /* Known value if no formula */
	    afield = afield -> next;
	  }

	afield = first_field -> next;
	while (afield != NULL)
	  { if (afield -> formula != NULL)
	      {	afield -> ucaseonly++;	/* We're calculating this field */
		afield -> value = parse(afield -> formula);
		if (Error)
			strcpy(afield->work,afield->mask);
		else
			update_value(afield);

		scrn_line = afield -> entry_row - cur_top;
		if (scrn_line >= scrn_top && scrn_line < scrn_bot)
		  { setcur(scrn_line + win_sr,afield->entry_col);
		    wstr(afield->work);
		  }
	      }
	    afield = afield -> next;
	  }
	stat_line = blanks;
	display_status();
}
